@@ -28,7 +28,9 @@ enum class PieceSet
 
     trigon,
 
-    nexos
+    nexos,
+
+    callisto
 };
 
 //-----------------------------------------------------------------------------
@@ -43,7 +45,13 @@ enum class BoardType
 
     trigon_3,
 
-    nexos
+    nexos,
+
+    callisto,
+
+    callisto_2,
+
+    callisto_3,
 };
 
 //-----------------------------------------------------------------------------
@@ -69,7 +77,13 @@ enum class Variant
 
     nexos,
 
-    nexos_2
+    nexos_2,
+
+    callisto,
+
+    callisto_2,
+
+    callisto_3
 };
 
 //-----------------------------------------------------------------------------
@@ -125,6 +139,10 @@ void get_transforms(Variant variant,
                     vector<unique_ptr<PointTransform<Point>>>& transforms,
                     vector<unique_ptr<PointTransform<Point>>>& inv_transforms);
 
+/** Is the variant a two-player variant with the board including the starting
+    points invariant through point reflection through its center? */
+bool has_central_symmetry(Variant variant);
+
 //-----------------------------------------------------------------------------
 
 } // namespace libpentobi_base
