@@ -84,9 +84,9 @@ public:
 
     void start_simulation(size_t n);
 
-    bool gen_children(Tree::NodeExpander& expander, Float init_val);
+    bool gen_children(Tree::NodeExpander& expander, Float root_val);
 
-    void start_playout();
+    void start_playout() { };
 
     /** Generate a playout move.
         @return @c false if end of game was reached, and no move was
@@ -123,11 +123,9 @@ private:
 
     Color::IntType m_nu_colors;
 
-    PieceSet m_piece_set;
+    BoardConst::MoveInfoArray m_move_info_array;
 
-    const MoveInfo* m_move_info_array;
-
-    const MoveInfoExt* m_move_info_ext_array;
+    BoardConst::MoveInfoExtArray m_move_info_ext_array;
 
     /** Incrementally updated lists of legal moves for both colors.
         Only the move list for the color to play van be used in any given
@@ -189,6 +187,8 @@ private:
         color individually. */
     bool m_force_consider_all_pieces;
 
+    bool m_is_callisto;
+
     /** Minimum number of pieces on board to perform a symmetry check.
         3 in Duo/Junior or 5 in Trigon because this is the earliest move number
         to break the symmetry. The early playout termination that evaluates all
@@ -198,6 +198,9 @@ private:
         symmetry can be broken a few moves later. */
     unsigned m_symmetry_min_nu_pieces;
 
+    /** Cache of m_bc->get_max_piece_size() */
+    unsigned m_max_piece_size;
+
     /** Remember attach points that were already used for move generation.
         Allows the incremental update of the move lists to skip attach points
         of newly played pieces that were already attach points of previously
@@ -205,80 +208,131 @@ private:
     ColorMap<Grid<bool>> m_moves_added_at;
 
 
-    template<unsigned PIECE_SIZE>
+    template<unsigned MAX_SIZE, bool IS_CALLISTO>
     void add_moves(Point p, Color c, const Board::PiecesLeftList& pieces,
                    float& total_gamma, MoveList& moves, unsigned& nu_moves);
 
-    template<unsigned PIECE_SIZE>
+    template<unsigned MAX_SIZE>
     LIBBOARDGAME_NOINLINE
     void add_starting_moves(Color c, const Board::PiecesLeftList& pieces,
                             bool with_gamma, MoveList& moves);
 
-    void evaluate_duo(array<Float, 6>& result);
+    template<unsigned MAX_SIZE>
+    LIBBOARDGAME_NOINLINE
+    void add_one_piece_moves(Color c, bool with_gamma, float& total_gamma,
+                             MoveList& moves, unsigned& nu_moves);
 
     void evaluate_multicolor(array<Float, 6>& result);
 
     void evaluate_multiplayer(array<Float, 6>& result);
 
+    void evaluate_twocolor(array<Float, 6>& result);
+
     Point find_best_starting_point(Color c) const;
 
-    Float get_quality_bonus(Color c, Float result, Float score,
-                            bool use_nu_attach);
+    Float get_quality_bonus(Color c, Float result, Float score);
 
-    /** Equivalent to but faster than m_bd.get_move_info() */
-    const MoveInfo& get_move_info(Move move) const;
+    Float get_quality_bonus_attach_twocolor();
 
-    /** Equivalent to but faster than m_bd.get_move_info_ext() */
-    const MoveInfoExt& get_move_info_ext(Move move) const;
+    Float get_quality_bonus_attach_multicolor();
+
+    template<unsigned MAX_SIZE>
+    const MoveInfo<MAX_SIZE>& get_move_info(Move mv) const;
+
+    template<unsigned MAX_ADJ_ATTACH>
+    const MoveInfoExt<MAX_ADJ_ATTACH>& get_move_info_ext(Move mv) const;
 
     PrecompMoves::Range get_moves(Color c, Piece piece, Point p,
                                   unsigned adj_status) const;
 
     bool has_moves(Color c, Piece piece, Point p, unsigned adj_status) const;
 
-    const PieceMap<bool>& get_is_piece_considered() const;
+    const PieceMap<bool>& get_is_piece_considered(Color c) const;
 
     const Board::PiecesLeftList& get_pieces_considered(Color c);
 
-    template<unsigned PIECE_SIZE>
+    template<unsigned MAX_SIZE, unsigned MAX_ADJ_ATTACH, bool IS_CALLISTO>
     void init_moves_with_gamma(Color c);
 
-    template<unsigned PIECE_SIZE>
+    template<unsigned MAX_SIZE>
     void init_moves_without_gamma(Color c);
 
-    template<unsigned PIECE_SIZE>
+    template<unsigned MAX_SIZE>
     bool check_forbidden(const GridExt<bool>& is_forbidden, Move mv,
                          MoveList& moves, unsigned& nu_moves);
 
-    template<unsigned PIECE_SIZE>
-    bool check_move(Move mv, const MoveInfo& info, float gamma_piece,
+    bool check_lgr(Move mv) const;
+
+    template<unsigned MAX_SIZE, bool IS_CALLISTO>
+    bool check_move(Move mv, const MoveInfo<MAX_SIZE>& info, float gamma_piece,
                     MoveList& moves, unsigned& nu_moves,
                     const PlayoutFeatures& playout_features,
                     float& total_gamma);
 
-    template<unsigned PIECE_SIZE>
-    bool check_move(Move mv, const MoveInfo& info, MoveList& moves,
+    template<unsigned MAX_SIZE, bool IS_CALLISTO>
+    bool check_move(Move mv, const MoveInfo<MAX_SIZE>& info, MoveList& moves,
                     unsigned& nu_moves,
                     const PlayoutFeatures& playout_features,
                     float& total_gamma);
 
     bool gen_playout_move_full(PlayerMove<Move>& mv);
 
-    template<unsigned PIECE_SIZE>
+    template<unsigned MAX_SIZE, unsigned MAX_ADJ_ATTACH, bool IS_CALLISTO>
     void update_moves(Color c);
 
+    template<unsigned MAX_SIZE, unsigned MAX_ADJ_ATTACH>
     void update_playout_features(Color c, Move mv);
 
-    void update_symmetry_broken(Move mv);
+    template<unsigned MAX_SIZE>
+    LIBBOARDGAME_NOINLINE void update_symmetry_broken(Move mv);
 };
 
+/** Check if last-good-reply move is applicable.
+    To be faster, it doesn't check for starting moves because such moves rarely
+    occur in the playout phase and doesn't check if a 1-piece move is in the
+    center in Callisto because such moves are not generated in the search. */
+inline bool State::check_lgr(Move mv) const
+{
+    if (mv.is_null())
+        return false;
+    Color c = m_bd.get_to_play();
+    auto piece = m_bd.get_move_piece(mv);
+    if (! m_bd.is_piece_left(c, piece))
+        return false;
+    auto points = m_bd.get_move_points(mv);
+    auto i = points.begin();
+    auto end = points.end();
+    bool has_attach_point = false;
+    do
+    {
+        if (m_bd.is_forbidden(*i, c))
+            return false;
+        // Logically, we mean:
+        // has_attach_point = has_attach_point || is_attach_point(*i, c)
+        // But this generates branches, which are bad for performance in this
+        // tight loop (unrolled by the compiler). So we use a bitwise OR, which
+        // works because C++ guarantees that true/false converts to 1/0.
+        has_attach_point |= m_bd.is_attach_point(*i, c);
+    }
+    while (++i != end);
+    if (m_is_callisto)
+    {
+        Piece one_piece = m_bd.get_one_piece();
+        if (piece == one_piece)
+            return true;
+        if (m_bd.get_nu_left_piece(c, one_piece) > 1 && piece != one_piece)
+            return false;
+    }
+    return has_attach_point;
+}
+
 inline void State::evaluate_playout(array<Float, 6>& result)
 {
     auto nu_players = m_bd.get_nu_players();
     if (nu_players == 2)
     {
         if (m_nu_colors == 2)
-            evaluate_duo(result);
+            evaluate_twocolor(result);
         else
             evaluate_multicolor(result);
     }
@@ -291,8 +345,7 @@ inline void State::finish_in_tree()
     if (log_simulations)
         LIBBOARDGAME_LOG("Finish in-tree");
     if (m_check_symmetric_draw)
-        m_is_symmetry_broken =
-            check_symmetry_broken(m_bd, m_shared_const.symmetric_points);
+        m_is_symmetry_broken = check_symmetry_broken(m_bd);
 }
 
 inline bool State::gen_playout_move(const LastGoodReply& lgr, Move last,
@@ -310,9 +363,7 @@ inline bool State::gen_playout_move(const LastGoodReply& lgr, Move last,
     }
     PlayerInt player = get_player();
     Move lgr2 = lgr.get_lgr2(player, last, second_last);
-    // We use Board::is_legal_nonfirst() because it is faster and smaller
-    // and the cases that the lgr move is the first move of a color is rare.
-    if (! lgr2.is_null() && m_bd.is_legal_nonfirst(m_bd.get_to_play(), lgr2))
+    if (check_lgr(lgr2))
     {
         if (log_simulations)
             LIBBOARDGAME_LOG("Playing last good reply 2");
@@ -320,7 +371,7 @@ inline bool State::gen_playout_move(const LastGoodReply& lgr, Move last,
         return true;
     }
     Move lgr1 = lgr.get_lgr1(player, last);
-    if (! lgr1.is_null() && m_bd.is_legal_nonfirst(m_bd.get_to_play(), lgr1))
+    if (check_lgr(lgr1))
     {
         if (log_simulations)
             LIBBOARDGAME_LOG("Playing last good reply 1");
@@ -330,18 +381,20 @@ inline bool State::gen_playout_move(const LastGoodReply& lgr, Move last,
     return gen_playout_move_full(mv);
 }
 
-inline const MoveInfo& State::get_move_info(Move mv) const
+template<unsigned MAX_SIZE>
+inline const MoveInfo<MAX_SIZE>& State::get_move_info(Move mv) const
 {
-    LIBBOARDGAME_ASSERT(! mv.is_null());
     LIBBOARDGAME_ASSERT(mv.to_int() < m_bc->get_nu_moves());
-    return *(m_move_info_array + mv.to_int());
+    return BoardConst::get_move_info<MAX_SIZE>(mv, m_move_info_array);
 }
 
-inline const MoveInfoExt& State::get_move_info_ext(Move mv) const
+template<unsigned MAX_ADJ_ATTACH>
+inline const MoveInfoExt<MAX_ADJ_ATTACH>& State::get_move_info_ext(
+        Move mv) const
 {
-    LIBBOARDGAME_ASSERT(! mv.is_null());
     LIBBOARDGAME_ASSERT(mv.to_int() < m_bc->get_nu_moves());
-    return *(m_move_info_ext_array + mv.to_int());
+    return BoardConst::get_move_info_ext<MAX_ADJ_ATTACH>(
+                mv, m_move_info_ext_array);
 }
 
 inline PrecompMoves::Range State::get_moves(Color c, Piece piece, Point p,
@@ -370,14 +423,27 @@ inline void State::play_in_tree(Move mv)
     if (! mv.is_null())
     {
         LIBBOARDGAME_ASSERT(m_bd.is_legal(to_play, mv));
-        m_bd.play(to_play, mv);
         m_nu_passes = 0;
-        update_playout_features(to_play, mv);
+        if (m_max_piece_size == 5)
+        {
+            m_bd.play<5, 16>(to_play, mv);
+            update_playout_features<5, 16>(to_play, mv);
+        }
+        else if (m_max_piece_size == 6)
+        {
+            m_bd.play<6, 22>(to_play, mv);
+            update_playout_features<6, 22>(to_play, mv);
+        }
+        else
+        {
+            m_bd.play<7, 12>(to_play, mv);
+            update_playout_features<7, 12>(to_play, mv);
+        }
     }
     else
     {
-        m_bd.set_to_play(to_play.get_next(m_nu_colors));
         ++m_nu_passes;
+        m_bd.set_to_play(to_play.get_next(m_nu_colors));
     }
     if (log_simulations)
         LIBBOARDGAME_LOG(m_bd);
@@ -387,19 +453,31 @@ inline void State::play_playout(Move mv)
 {
     auto to_play = m_bd.get_to_play();
     LIBBOARDGAME_ASSERT(m_bd.is_legal(to_play, mv));
-    m_bd.play(to_play, mv);
+    if (m_max_piece_size == 5)
+    {
+        m_bd.play<5, 16>(to_play, mv);
+        update_playout_features<5, 16>(to_play, mv);
+        if (! m_is_symmetry_broken)
+            update_symmetry_broken<5>(mv);
+    }
+    else if (m_max_piece_size == 6)
+    {
+        m_bd.play<6, 22>(to_play, mv);
+        update_playout_features<6, 22>(to_play, mv);
+        if (! m_is_symmetry_broken)
+            update_symmetry_broken<6>(mv);
+    }
+    else
+    {
+        m_bd.play<7, 12>(to_play, mv);
+        update_playout_features<7, 12>(to_play, mv);
+        // No game variant with piece size 7 uses m_is_symmetry_broken
+    }
     ++m_nu_new_moves[to_play];
     m_last_move[to_play] = mv;
     m_nu_passes = 0;
-    if (! m_is_symmetry_broken)
-        update_symmetry_broken(mv);
     if (log_simulations)
         LIBBOARDGAME_LOG(m_bd);
-    update_playout_features(to_play, mv);
-}
-
-inline void State::start_playout()
-{
 }
 
 inline bool State::skip_rave(Move mv) const
@@ -408,12 +486,56 @@ inline bool State::skip_rave(Move mv) const
     return false;
 }
 
+template<unsigned MAX_SIZE, unsigned MAX_ADJ_ATTACH>
 inline void State::update_playout_features(Color c, Move mv)
 {
-    auto& info = get_move_info(mv);
+    auto& info = get_move_info<MAX_SIZE>(mv);
     for (Color i : Color::Range(m_nu_colors))
         m_playout_features[i].set_forbidden(info);
-    m_playout_features[c].set_forbidden(get_move_info_ext(mv));
+    m_playout_features[c].set_forbidden<MAX_ADJ_ATTACH>(
+                get_move_info_ext<MAX_ADJ_ATTACH>(mv));
+}
+
+template<unsigned MAX_SIZE>
+void State::update_symmetry_broken(Move mv)
+{
+    Color to_play = m_bd.get_to_play();
+    Color second_color = m_bd.get_second_color(to_play);
+    auto& symmetric_points = m_bc->get_symmetrc_points();
+    auto& info = get_move_info<MAX_SIZE>(mv);
+    auto i = info.begin();
+    auto end = info.end();
+    if (to_play == Color(0) || to_play == Color(2))
+    {
+        // First player to play: Check that all symmetric points of the last
+        // move of the second player are occupied by the first player
+        do
+        {
+            Point symm_p = symmetric_points[*i];
+            if (m_bd.get_point_state(symm_p) != second_color)
+            {
+                m_is_symmetry_broken = true;
+                return;
+            }
+        }
+        while (++i != end);
+    }
+    else
+    {
+        // Second player to play: Check that all symmetric points of the last
+        // move of the first player are empty (i.e. the second player can play
+        // there to preserve the symmetry)
+        do
+        {
+            Point symm_p = symmetric_points[*i];
+            if (! m_bd.get_point_state(symm_p).is_empty())
+            {
+                m_is_symmetry_broken = true;
+                return;
+            }
+        }
+        while (++i != end);
+    }
 }
 
 //-----------------------------------------------------------------------------
