@@ -10,6 +10,8 @@
 
 #include "AnalyzeGameWidget.h"
 
+#include <QApplication>
+#include <QDesktopWidget>
 #include <QLabel>
 #include <QMouseEvent>
 #include <QProgressDialog>
@@ -74,7 +76,9 @@ void AnalyzeGameWidget::paintEvent(QPaintEvent*)
     QPainter painter(this);
     QFont font;
     font.setStyleStrategy(QFont::PreferOutline);
-    font.setPixelSize(static_cast<int>(0.06 * height()));
+    // QFont::setPixelSize(0) prints a warning even if it works and the docs
+    // of Qt 5.3 don't forbid it (unlike QFont::setPointSize(0)).
+    font.setPixelSize(max(1, static_cast<int>(0.06 * height())));
     QFontMetrics metrics(font);
     painter.translate(m_borderX, m_borderY);
     painter.setPen(Qt::NoPen);
@@ -112,8 +116,6 @@ void AnalyzeGameWidget::paintEvent(QPaintEvent*)
     painter.setRenderHint(QPainter::Antialiasing, true);
     for (unsigned i = 0; i < nu_moves; ++i)
     {
-        if (! m_analyzeGame.has_value(i))
-            continue;
         double value = m_analyzeGame.get_value(i);
         // Values can be outside [0..1] due to score/length bonuses
         if (value < 0)
@@ -174,7 +176,8 @@ void AnalyzeGameWidget::showProgress(int progress)
 
 QSize AnalyzeGameWidget::sizeHint() const
 {
-    return QSize(800, 240);
+    auto geo = QApplication::desktop()->screenGeometry();
+    return QSize(geo.width() / 2, geo.height() / 3);
 }
 
 void AnalyzeGameWidget::start(const Game& game, Search& search,
