@@ -49,8 +49,8 @@
         <translation>Über Pentobi</translation>
     </message>
     <message>
-        <source>Select Next &amp;Color</source>
-        <translation>Nächste &amp;Farbe selektieren</translation>
+        <source>Next &amp;Color</source>
+        <translation>Nächste &amp;Farbe</translation>
     </message>
     <message>
         <source>S&amp;etup Mode</source>
@@ -454,10 +454,6 @@
         <source>&amp;Computer</source>
         <translation>&amp;Computer</translation>
     </message>
-    <message>
-        <source>&amp;Level</source>
-        <translation>S&amp;pielstufe</translation>
-    </message>
     <message>
         <source>&amp;Tools</source>
         <translation>E&amp;xtras</translation>
@@ -708,31 +704,10 @@
         <source>Setup mode</source>
         <translation>Stellungsaufbau</translation>
     </message>
-    <message numerus="yes">
-        <source>Blue wins with %n point(s).</source>
-        <translation>
-            <numerusform>Blau gewinnt mit %n Punkt.</numerusform>
-            <numerusform>Blau gewinnt mit %n Punkten.</numerusform>
-        </translation>
-    </message>
     <message>
         <source>The game ends in a tie.</source>
         <translation>Das Spiel endet in einem Unentschieden.</translation>
     </message>
-    <message numerus="yes">
-        <source>Blue/Red wins with %n point(s).</source>
-        <translation>
-            <numerusform>Blau/Rot gewinnt mit %n Punkt.</numerusform>
-            <numerusform>Blau/Rot gewinnt mit %n Punkten.</numerusform>
-        </translation>
-    </message>
-    <message numerus="yes">
-        <source>Yellow/Green wins with %n point(s).</source>
-        <translation>
-            <numerusform>Gelb/Grün gewinnt mit %n Punkt.</numerusform>
-            <numerusform>Gelb/Grün gewinnt mit %n Punkten.</numerusform>
-        </translation>
-    </message>
     <message>
         <source>The game ends in a tie between all colors.</source>
         <translation>Das Spiel endet in einem Unentschieden zwischen allen Farben.</translation>
@@ -781,13 +756,6 @@
         <source>Blue wins.</source>
         <translation>Blau gewinnt.</translation>
     </message>
-    <message numerus="yes">
-        <source>Green wins with %n point(s).</source>
-        <translation>
-            <numerusform>Grün gewinnt mit %n Punkt.</numerusform>
-            <numerusform>Grün gewinnt mit %n Punkten.</numerusform>
-        </translation>
-    </message>
     <message>
         <source>Yellow wins.</source>
         <translation>Gelb gewinnt.</translation>
@@ -848,8 +816,8 @@
         <translation>Klassisch (&amp;4 Spieler)</translation>
     </message>
     <message>
-        <source>&amp;Junior</source>
-        <translation>&amp;Junior</translation>
+        <source>J&amp;unior</source>
+        <translation>J&amp;unior</translation>
     </message>
     <message>
         <source>Classic (&amp;2 Players)</source>
@@ -887,6 +855,122 @@
         <source>&amp;Nexos</source>
         <translation>&amp;Nexos</translation>
     </message>
+    <message>
+        <source>Blue wins with 1 point.</source>
+        <translation>Blau gewinnt mit 1 Punkt.</translation>
+    </message>
+    <message>
+        <source>Blue wins with %1 points.</source>
+        <translation>Blau gewinnt mit %1 Punkten.</translation>
+    </message>
+    <message>
+        <source>Green wins with 1 point.</source>
+        <translation>Grün gewinnt mit 1 Punkt.</translation>
+    </message>
+    <message>
+        <source>Green wins with %1 points.</source>
+        <translation>Grün gewinnt mit %1 Punkten.</translation>
+    </message>
+    <message>
+        <source>Blue/Red wins with 1 point.</source>
+        <translation>Blau/Rot gewinnt mit 1 Punkt.</translation>
+    </message>
+    <message>
+        <source>Blue/Red wins with %1 points.</source>
+        <translation>Blau/Rot gewinnt mit %1 Punkten.</translation>
+    </message>
+    <message>
+        <source>Yellow/Green wins with 1 point.</source>
+        <translation>Gelb/Grün gewinnt mit 1 Punkt.</translation>
+    </message>
+    <message>
+        <source>Yellow/Green wins with %1 points.</source>
+        <translation>Gelb/Grün gewinnt mit %1 Punkten.</translation>
+    </message>
+    <message>
+        <source>Callisto (&amp;2 Players)</source>
+        <translation>Callisto (&amp;2 Spieler)</translation>
+    </message>
+    <message>
+        <source>Callisto (&amp;3 Players)</source>
+        <translation>Callisto (&amp;3 Spieler)</translation>
+    </message>
+    <message>
+        <source>Callisto (&amp;4 Players)</source>
+        <translation>Callisto (&amp;4 Spieler)</translation>
+    </message>
+    <message>
+        <source>C&amp;allisto</source>
+        <translation>&amp;Callisto</translation>
+    </message>
+    <message>
+        <source>Green wins (tie resolved).</source>
+        <translation>Grün gewinnt (Unentschieden aufgelöst).</translation>
+    </message>
+    <message>
+        <source>Yellow/Green wins (tie resolved).</source>
+        <translation>Gelb/Grün gewinnt (Unentschieden aufgelöst).</translation>
+    </message>
+    <message>
+        <source>Red wins (tie resolved).</source>
+        <translation>Rot gewinnt (Unentschieden aufgelöst).</translation>
+    </message>
+    <message>
+        <source>Yellow wins (tie resolved).</source>
+        <translation>Gelb gewinnt (Unentschieden aufgelöst).</translation>
+    </message>
+    <message>
+        <source>&amp;Level (Classic, 4 Players)</source>
+        <translation>Spielst&amp;ufe (Klassisch, 4 Spieler)</translation>
+    </message>
+    <message>
+        <source>&amp;Level (Classic, 2 Players)</source>
+        <translation>Spielst&amp;ufe (Klassisch, 2 Spieler)</translation>
+    </message>
+    <message>
+        <source>&amp;Level (Classic, 3 Players)</source>
+        <translation>Spielst&amp;ufe (Klassisch, 3 Spieler)</translation>
+    </message>
+    <message>
+        <source>&amp;Level (Duo)</source>
+        <translation>Spielst&amp;ufe (Duo)</translation>
+    </message>
+    <message>
+        <source>&amp;Level (Trigon, 4 Players)</source>
+        <translation>Spielst&amp;ufe (Trigon, 4 Spieler)</translation>
+    </message>
+    <message>
+        <source>&amp;Level (Trigon, 2 Players)</source>
+        <translation>Spielst&amp;ufe (Trigon, 2 Spieler)</translation>
+    </message>
+    <message>
+        <source>&amp;Level (Trigon, 3 Players)</source>
+        <translation>Spielst&amp;ufe (Trigon, 3 Spieler)</translation>
+    </message>
+    <message>
+        <source>&amp;Level (Junior)</source>
+        <translation>Spielst&amp;ufe (Junior)</translation>
+    </message>
+    <message>
+        <source>&amp;Level (Nexos, 4 Players)</source>
+        <translation>Spielst&amp;ufe (Nexos, 4 Spieler)</translation>
+    </message>
+    <message>
+        <source>&amp;Level (Nexos, 2 Players)</source>
+        <translation>Spielst&amp;ufe (Nexos, 2 Spieler)</translation>
+    </message>
+    <message>
+        <source>&amp;Level (Callisto, 4 Players)</source>
+        <translation>Spielst&amp;ufe (Callisto, 4 Spieler)</translation>
+    </message>
+    <message>
+        <source>&amp;Level (Callisto, 2 Players)</source>
+        <translation>Spielst&amp;ufe (Callisto, 2 Spieler)</translation>
+    </message>
+    <message>
+        <source>&amp;Level (Callisto, 3 Players)</source>
+        <translation>Spielst&amp;ufe (Callisto, 3 Spieler)</translation>
+    </message>
 </context>
 <context>
     <name>RatedGamesList</name>
@@ -1005,6 +1089,18 @@
         <source>Nexos (2 players)</source>
         <translation>Nexos (2 Spieler)</translation>
     </message>
+    <message>
+        <source>Callisto (4 players)</source>
+        <translation>Callisto (4 Spieler)</translation>
+    </message>
+    <message>
+        <source>Callisto (2 players)</source>
+        <translation>Callisto (2 Spieler)</translation>
+    </message>
+    <message>
+        <source>Callisto (3 players)</source>
+        <translation>Callisto (3 Spieler)</translation>
+    </message>
 </context>
 <context>
     <name>main</name>
