@@ -1,17 +1,3 @@
-function clearMarks() {
-    if (_pieces0 === undefined) // Pieces not yet created
-        return
-    var i
-    for (i = 0; i < _pieces0.length; ++i)
-        _pieces0[i].isMarked = false
-    for (i = 0; i < _pieces1.length; ++i)
-        _pieces1[i].isMarked = false
-    for (i = 0; i < _pieces2.length; ++i)
-        _pieces2[i].isMarked = false
-    for (i = 0; i < _pieces3.length; ++i)
-        _pieces3[i].isMarked = false
-}
-
 function createColorPieces(component, pieceModels) {
     if (pieceModels.length === 0)
         return []
@@ -19,14 +5,19 @@ function createColorPieces(component, pieceModels) {
     switch (pieceModels[0].color) {
     case 0: colorName = "blue"; break
     case 1:
-        colorName = gameModel.gameVariant == "duo" ||
-                gameModel.gameVariant == "junior" ? "green" : "yellow"; break
+        colorName = gameModel.gameVariant == "duo"
+                || gameModel.gameVariant == "junior"
+                || gameModel.gameVariant == "callisto_2" ?
+                    "green" : "yellow"; break
     case 2: colorName = "red"; break
     case 3: colorName = "green"; break
     }
     var properties = {
         "colorName": colorName,
-        "isPicked": Qt.binding(function() { return this === pickedPiece }) }
+        "isPicked": Qt.binding(function() { return this === pickedPiece }),
+        "isMarked": Qt.binding(function() {
+            return markLastMove && this.pieceModel.isLastMove })
+    }
     var pieces = []
     for (var i = 0; i < pieceModels.length; ++i) {
         properties["pieceModel"] = pieceModels[i]
@@ -36,46 +27,49 @@ function createColorPieces(component, pieceModels) {
 }
 
 function createPieces() {
-    var file = (gameModel.gameVariant.indexOf("trigon") >= 0) ?
-                "PieceTrigon.qml" : "PieceClassic.qml"
+    var file
+    if (gameModel.gameVariant.indexOf("trigon") === 0)
+        file = "PieceTrigon.qml"
+    else if (gameModel.gameVariant.indexOf("nexos") === 0)
+        file = "PieceNexos.qml"
+    else if (gameModel.gameVariant.indexOf("callisto") === 0)
+        file = "PieceCallisto.qml"
+    else
+        file = "PieceClassic.qml"
     var component = Qt.createComponent(file)
-    if (component.status !== Component.Ready)
-        throw "Could not create component " + file
-    _pieces0 = createColorPieces(component, gameModel.pieceModels0)
-    _pieces1 = createColorPieces(component, gameModel.pieceModels1)
-    _pieces2 = createColorPieces(component, gameModel.pieceModels2)
-    _pieces3 = createColorPieces(component, gameModel.pieceModels3)
-    if (markLastMove)
-        markLast()
+    pieces0 = createColorPieces(component, gameModel.pieceModels0)
+    pieces1 = createColorPieces(component, gameModel.pieceModels1)
+    pieces2 = createColorPieces(component, gameModel.pieceModels2)
+    pieces3 = createColorPieces(component, gameModel.pieceModels3)
+    pieceSelector.transitionsEnabled =
+            Qt.binding(function() { return enableAnimations })
 }
 
 function destroyColorPieces(pieces) {
     if (pieces === undefined)
         return
     for (var i = 0; i < pieces.length; ++i) {
-        // Not sure why setting visible to false is necessary before destroy,
-        // but otherwise some on-board pieces sometimes don't disappear
-        // (happened with Qt 5.5/Linux when loading a Trigon game with pieces
-        // on board and then switching to game variant Classic).
         pieces[i].visible = false
-        pieces[i].destroy()
+        pieces[i].destroy(1000)
     }
 }
 
 function destroyPieces() {
-    destroyColorPieces(_pieces0); _pieces0 = []
-    destroyColorPieces(_pieces1); _pieces1 = []
-    destroyColorPieces(_pieces2); _pieces2 = []
-    destroyColorPieces(_pieces3); _pieces3 = []
+    pieceSelector.transitionsEnabled = false
+    pickedPiece = null
+    destroyColorPieces(pieces0); pieces0 = []
+    destroyColorPieces(pieces1); pieces1 = []
+    destroyColorPieces(pieces2); pieces2 = []
+    destroyColorPieces(pieces3); pieces3 = []
 }
 
 function findPiece(pieceModel, color) {
     var pieces
     switch (color) {
-    case 0: pieces = _pieces0; break
-    case 1: pieces = _pieces1; break
-    case 2: pieces = _pieces2; break
-    case 3: pieces = _pieces3; break
+    case 0: pieces = pieces0; break
+    case 1: pieces = pieces1; break
+    case 2: pieces = pieces2; break
+    case 3: pieces = pieces3; break
     }
     if (pieces === undefined)
         return null // Pieces haven't been created yet
@@ -85,14 +79,6 @@ function findPiece(pieceModel, color) {
     return null
 }
 
-function markLast() {
-    var piece = findPiece(gameModel.getLastMovePieceModel(),
-                          gameModel.getLastMoveColor())
-    if (piece === null)
-        return
-    piece.isMarked = true
-}
-
 function pickPiece(piece) {
     if (playerModel.isGenMoveRunning || gameModel.isGameOver
             || piece.pieceModel.color !== gameModel.toPlay)
@@ -108,26 +94,15 @@ function pickPiece(piece) {
         pieceManipulator.x = x
         pieceManipulator.y = y
     }
-    transitionsEnabled = true
     pickedPiece = piece
 }
 
-function positionChanged() {
-    dropPiece()
-    clearMarks()
-    if (markLastMove)
-        markLast()
-}
-
 function showMoveHint(move) {
     var pieceModel = gameModel.preparePiece(gameModel.toPlay, move)
-    var piece = findPiece(pieceModel, gameModel.toPlay)
     var pos = board.mapToItem(pieceManipulator.parent,
                               board.mapFromGameX(pieceModel.gameCoord.x),
                               board.mapFromGameY(pieceModel.gameCoord.y))
     pieceManipulator.x = pos.x - pieceManipulator.width / 2
     pieceManipulator.y = pos.y - pieceManipulator.height / 2
-    transitionsEnabled = true
-    pickedPiece = piece
-    transitionsEnabled = false
+    pickedPiece = findPiece(pieceModel, gameModel.toPlay)
 }
