@@ -1,27 +1,37 @@
-function cancelGenMove() {
-    playerModel.cancelGenMove()
-    gameDisplay.busyIndicatorRunning = false
+function about() {
+    var url = "http://pentobi.sourceforge.net"
+    showInfo("<h2>" + qsTr("Pentobi") + "</h2><p>" +
+             qsTr("Version %1").arg(Qt.application.version) + "</p><p>" +
+             qsTr("Computer opponent for the board game Blokus.") + "<br>" +
+             qsTr("&copy; 2011&ndash;%1 Markus&nbsp;Enzenberger").arg(2016) +
+             "<br><a href=\"" + url + "\">" + url + "</a></p>")
 }
 
-function changeGameVariant(gameVariant, verifyAbortGame) {
+function changeGameVariant(gameVariant) {
     if (gameModel.gameVariant === gameVariant)
         return
-    cancelGenMove()
-    if (! gameModel.isGameEmpty && ! gameModel.isGameOver &&
-            verifyAbortGame) {
+    if (! gameModel.isGameEmpty && ! gameModel.isGameOver) {
         showQuestion(qsTr("New game?"),
-                     function() { changeGameVariant(gameVariant, false) })
+                     function() { changeGameVariantNoVerify(gameVariant) })
         return
     }
-    callDelayTimer.call(function() {
-        initGameVariant(gameVariant)
+    changeGameVariantNoVerify(gameVariant)
+}
+
+function changeGameVariantNoVerify(gameVariant) {
+    cancelGenMove()
+    lengthyCommand.run(function() {
+        gameDisplay.destroyPieces()
+        gameModel.initGameVariant(gameVariant)
+        gameDisplay.createPieces()
+        gameDisplay.showToPlay()
         initComputerColors()
     })
 }
 
 function checkComputerMove() {
     if (gameModel.isGameOver) {
-        showGameOver()
+        showInfo(gameModel.getResultMessage())
         return
     }
     if (! isComputerToPlay())
@@ -41,7 +51,6 @@ function checkComputerMove() {
 function computerPlay() {
     if (playerModel.isGenMoveRunning)
         return
-    hideComputerColorDialog()
     if (! isComputerToPlay()) {
         computerPlays0 = false
         computerPlays1 = false
@@ -58,7 +67,8 @@ function computerPlay() {
         else
         {
             var isMultiColor =
-                    (variant == "classic_2" || variant == "trigon_2")
+                    (variant == "classic_2" || variant == "trigon_2"
+                     || variant == "nexos_2")
             switch (gameModel.toPlay) {
             case 0:
                 computerPlays0 = true
@@ -91,33 +101,17 @@ function computerPlays(color) {
     }
 }
 
-function computerPlaysAll() {
-    switch (gameModel.gameVariant) {
-    case "duo":
-    case "junior":
-    case "classic_2":
-    case "trigon_2":
-        return computerPlays0 && computerPlays1
-    case "trigon_3":
-        return computerPlays0 && computerPlays1 && computerPlays2
-    default:
-        return computerPlays0 && computerPlays1 && computerPlays2 &&
-                computerPlays3
-    }
-}
-
 function createTheme(themeName) {
     var source = "qrc:///qml/themes/" + themeName + "/Theme.qml"
-    var component = Qt.createComponent(source)
-    if (component.status !== Component.Ready)
-        throw "Could not load " + source
-    return component.createObject(root)
+    return Qt.createComponent(source).createObject(root)
+}
+
+function deleteAllVar() {
+    showQuestion(qsTr("Delete all variations?"), gameModel.deleteAllVar)
 }
 
 function genMove() {
-    cancelGenMove()
-    gameDisplay.dropPiece()
-    gameDisplay.busyIndicatorRunning = true
+    gameDisplay.pickedPiece = null
     isMoveHintRunning = false
     playerModel.startGenMove(gameModel)
 }
@@ -128,24 +122,27 @@ function getFileFromUrl(fileUrl) {
     return decodeURIComponent(file)
 }
 
-function hideComputerColorDialog()
-{
-    if (computerColorDialogLoader.status === Loader.Ready)
-        computerColorDialogLoader.item.visible = false
-}
-
 function init() {
-    callDelayTimer.call(function() {
-        if (! gameModel.loadAutoSave()) {
-            gameDisplay.createPieces()
-            initComputerColors()
+    // Settings might contain unusable geometry
+    var maxWidth = Screen.desktopAvailableWidth
+    var maxHeight = Screen.desktopAvailableHeight
+    if (x < 0 || x + width > maxWidth || y < 0 || y + height > maxHeight) {
+        if (width > maxWidth || height > Screen.maxHeight) {
+            width = defaultWidth
+            height = defaultHeight
         }
-        else {
-            gameDisplay.createPieces()
-            if (! computerPlaysAll())
-                checkComputerMove()
-        }
-    })
+        x = (maxWidth - width) / 2
+        y = (maxHeight - height) / 2
+    }
+    if (! gameModel.loadAutoSave()) {
+        gameDisplay.createPieces()
+        initComputerColors()
+    }
+    else {
+        gameDisplay.createPieces()
+        if (! gameModel.isGameOver)
+            checkComputerMove()
+    }
 }
 
 function initComputerColors() {
@@ -155,19 +152,11 @@ function initComputerColors() {
     computerPlays2 = true
     computerPlays3 = true
     if (gameModel.gameVariant == "classic_2"
-            || gameModel.gameVariant == "trigon_2")
+            || gameModel.gameVariant == "trigon_2"
+            || gameModel.gameVariant == "nexos_2")
         computerPlays2 = false
 }
 
-function initGameVariant(gameVariant) {
-    cancelGenMove()
-    hideComputerColorDialog()
-    gameDisplay.destroyPieces()
-    gameModel.initGameVariant(gameVariant)
-    gameDisplay.createPieces()
-    gameDisplay.showToPlay()
-}
-
 function isComputerToPlay() {
     if (gameModel.gameVariant == "classic_3" && gameModel.toPlay == 3)
         return computerPlays(gameModel.altPlayer)
@@ -175,52 +164,41 @@ function isComputerToPlay() {
 }
 
 function moveGenerated(move) {
-    gameDisplay.busyIndicatorRunning = false
     if (isMoveHintRunning) {
         gameDisplay.showMoveHint(move)
         isMoveHintRunning = false
         return
     }
-    gameDisplay.transitionsEnabled = true
     gameModel.playMove(move)
-    gameDisplay.transitionsEnabled = false
-    delayedCheckComputerMove.start()
+    delayedCheckComputerMove.restart()
 }
 
 function moveHint() {
     if (gameModel.isGameOver)
         return
-    cancelGenMove()
     isMoveHintRunning = true
-    gameDisplay.busyIndicatorRunning = true
     playerModel.startGenMoveAtLevel(gameModel, 1)
 }
 
-function newGame(verifyAbortGame)
+function newGameNoVerify()
 {
-    cancelGenMove()
-    if (! gameModel.isGameEmpty &&  ! gameModel.isGameOver &&
-            verifyAbortGame) {
-        showQuestion(qsTr("New game?"), function() { newGame(false) })
-        return
-    }
-    gameDisplay.dropPiece()
-    hideComputerColorDialog()
     gameModel.newGame()
     gameDisplay.showToPlay()
     initComputerColors()
 }
 
-function open() {
-    if (openDialogLoader.status === Loader.Null)
-        openDialogLoader.sourceComponent = openDialogComponent
-    openDialogLoader.item.open()
+function newGame()
+{
+    if (! gameModel.isGameEmpty &&  ! gameModel.isGameOver) {
+        showQuestion(qsTr("New game?"), newGameNoVerify)
+        return
+    }
+    newGameNoVerify()
 }
 
-function openFileUrl(fileUrl) {
-    cancelGenMove()
+function openFileUrl() {
     gameDisplay.destroyPieces()
-    if (! gameModel.open(getFileFromUrl(fileUrl)))
+    if (! gameModel.open(getFileFromUrl(openDialog.item.fileUrl)))
         showError(qsTr("Open failed.") + "\n" + gameModel.lastInputOutputError)
     else {
         computerPlays0 = false
@@ -233,14 +211,12 @@ function openFileUrl(fileUrl) {
 }
 
 function play(pieceModel, gameCoord) {
-    cancelGenMove()
     var wasComputerToPlay = isComputerToPlay()
-    gameDisplay.transitionsEnabled = true
     gameModel.playPiece(pieceModel, gameCoord)
     // We don't continue automatic play if the human played a move for a color
     // played by the computer.
     if (! wasComputerToPlay)
-        delayedCheckComputerMove.start()
+        delayedCheckComputerMove.restart()
 }
 
 function saveFileUrl(fileUrl) {
@@ -248,12 +224,6 @@ function saveFileUrl(fileUrl) {
         showError(qsTr("Save failed.") + "\n" + gameModel.lastInputOutputError)
 }
 
-function saveAs() {
-    if (saveDialogLoader.status === Loader.Null)
-        saveDialogLoader.sourceComponent = saveDialogComponent
-    saveDialogLoader.item.open()
-}
-
 function showComputerColorDialog() {
     if (computerColorDialogLoader.status === Loader.Null)
         computerColorDialogLoader.sourceComponent =
@@ -266,77 +236,6 @@ function showComputerColorDialog() {
     dialog.open()
 }
 
-function showGameOver() {
-    if (! gameModel.isGameOver)
-        return
-    var msg, points0, points1, points2, points3
-    switch (gameModel.gameVariant) {
-    case "duo":
-    case "junior":
-        points0 = gameModel.points0
-        points1 = gameModel.points1
-        if (points0 > points1)
-            msg = qsTr("Blue wins.")
-        else if (points0 < points1)
-            msg = qsTr("Green wins.")
-        else
-            msg = qsTr("Game ends in a tie.")
-        break
-    case "classic_2":
-    case "trigon_2":
-        points0 = gameModel.points0 + gameModel.points2
-        points1 = gameModel.points1 + gameModel.points3
-        if (points0 > points1)
-            msg = qsTr("Blue/Red wins.")
-        else if (points0 < points1)
-            msg = qsTr("Yellow/Green wins.")
-        else
-            msg = qsTr("Game ends in a tie.")
-        break
-    case "classic_3":
-    case "trigon_3":
-        points0 = gameModel.points0
-        points1 = gameModel.points1
-        points2 = gameModel.points2
-        var maxPoints = Math.max(points0, points1, points2)
-        var nuWinners = 0
-        if (points0 === maxPoints) ++nuWinners
-        if (points1 === maxPoints) ++nuWinners
-        if (points2 === maxPoints) ++nuWinners
-        if (nuWinners > 1)
-            msg = qsTr("Game ends in a tie.")
-        else if (points0 === maxPoints)
-            msg = qsTr("Blue wins.")
-        else if (points1 === maxPoints)
-            msg = qsTr("Yellow wins.")
-        else if (points2 === maxPoints)
-            msg = qsTr("Red wins.")
-        break
-    default:
-        points0 = gameModel.points0
-        points1 = gameModel.points1
-        points2 = gameModel.points2
-        points3 = gameModel.points3
-        maxPoints = Math.max(points0, points1, points2, points3)
-        nuWinners = 0
-        if (points0 === maxPoints) ++nuWinners
-        if (points1 === maxPoints) ++nuWinners
-        if (points2 === maxPoints) ++nuWinners
-        if (points3 === maxPoints) ++nuWinners
-        if (nuWinners > 1)
-            msg = qsTr("Game ends in a tie.")
-        else if (points0 === maxPoints)
-            msg = qsTr("Blue wins.")
-        else if (points1 === maxPoints)
-            msg = qsTr("Yellow wins.")
-        else if (points2 === maxPoints)
-            msg = qsTr("Red wins.")
-        else if (points3 === maxPoints)
-            msg = qsTr("Green wins.")
-    }
-    showInfo(msg)
-}
-
 function showError(text) {
     if (errorMessageLoader.status === Loader.Null)
         errorMessageLoader.sourceComponent = errorMessageComponent
@@ -363,26 +262,13 @@ function showQuestion(text, acceptedFunc) {
 }
 
 function truncate() {
-    showQuestion(qsTr("Truncate this subtree?"),
-                 function() { gameModel.truncate() })
+    showQuestion(qsTr("Truncate this subtree?"), gameModel.truncate)
 }
 
 function truncateChildren() {
-    showQuestion(qsTr("Truncate children?"),
-                 function() { gameModel.truncateChildren() })
+    showQuestion(qsTr("Truncate children?"), gameModel.truncateChildren)
 }
 
 function undo() {
-    if (! gameModel.canUndo)
-        return
-    cancelGenMove()
-    gameDisplay.transitionsEnabled = true
     gameModel.undo()
-    gameDisplay.transitionsEnabled = false
-    gameDisplay.dropPiece()
-}
-
-function quit() {
-    cancelGenMove()
-    gameModel.autoSave()
 }
